﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("CalculateNumberOfDeletionsOperator", "Description missing")]
  [StorableClass]
  public class CalculateNumberOfDeletionsOperator : SingleSuccessorOperator {

    #region Parameter Properties
    public ILookupParameter<IntValue> PopulationSizeParameter {
      get { return (ILookupParameter<IntValue>)Parameters["PopulationSize"]; }
    }
    public ILookupParameter<IntValue> CurrentPopulationSizeParameter {
      get { return (ILookupParameter<IntValue>)Parameters["CurrentPopulationSize"]; }
    }
    public IValueLookupParameter<IntValue> NumberOfDeletionsParameter {
      get { return (IValueLookupParameter<IntValue>)Parameters["NumberOfDeletions"]; }
    }
    #endregion

    [StorableConstructor]
    protected CalculateNumberOfDeletionsOperator(bool deserializing) : base(deserializing) { }
    protected CalculateNumberOfDeletionsOperator(CalculateNumberOfDeletionsOperator original, Cloner cloner)
      : base(original, cloner) {
    }
    public CalculateNumberOfDeletionsOperator()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("CurrentPopulationSize"));
      Parameters.Add(new LookupParameter<IntValue>("PopulationSize"));
      Parameters.Add(new ValueLookupParameter<IntValue>("NumberOfDeletions"));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CalculateNumberOfDeletionsOperator(this, cloner);
    }

    public override IOperation Apply() {
      int numberOfDeletions = CurrentPopulationSizeParameter.ActualValue.Value - PopulationSizeParameter.ActualValue.Value;
      if (numberOfDeletions > 0) {
        NumberOfDeletionsParameter.ActualValue = new IntValue(numberOfDeletions);
      } else {
        NumberOfDeletionsParameter.ActualValue = new IntValue(0);
      }
      return base.Apply();
    }
  }
}
